# frozen_string_literal: true
# checker Api::Dmo::HermesController avant d'ajouter des méthodes
module Hermes
  module Api
    module Dmo
      class UserRightsController < Hermes::Api::Dmo::ApplicationController

        VALID_OPERATORS = %w[
          $in
          $nin
          $ne
          $gt
          $gte
          $lt
          $lte
          $regex
        ].freeze

        ACCEPTABLE_PARAMS = %w[
          right
          type
          title
          group_id
          entity_id
          lessor_id
          user_id
          user_email
          user_first_name
          user_last_name
          user_phone_number
          user_auth_roles
          user_gender
        ].freeze

        REGEX_PARAMS = %w[
        ].freeze

        MUTATE_PARAMS = %i[
          right
          type
          entity_id
          lessor_id
          user_id
          user_email
          email
          user_email
          user_first_name
          user_last_name
          user_phone_number
          user_auth_roles
          user_gender
        ].freeze
        
        def mutate_params
          MUTATE_PARAMS
        end 
        
        def right_on_entity(ur = nil, no_params = false, new_params = nil)
          return true if is_super_admin
          p = new_params.blank? ? no_params ? {} : mutate_valid_params : new_params
          type = p[:type] || ur&.type
          entity_id = p[:entity_id] || ur&.entity_id
          lessor_id = p[:lessor_id] || ur&.lessor_id

          entity = (type == 'entity') ? Hermes::Entity.find(entity_id) : Hermes::Lessor.find(lessor_id)

          current_user.group_ids.to_array.include?(entity.manager_group_id)
        end


        def create_without_render
          @instance = @model.new(mutate_valid_params)
        end

        def set_instance
          @instance = @model.find_by(id: params[:id], **@query_precision) if params[:id]
        end

        def create
          return render_json_error(:unauthorized, :no_right_on_entity) unless right_on_entity
          return render_json_error(:unprocessable_entity, :missing_user_email) unless params[:data][:attributes][:user_email]

          super
        end

        def update
          return render_json_error(:not_found, :instance_not_found) unless @instance
          # return render_error_not_mutable(not_mutable) unless not_mutable.empty?
          return render_json_error(:unauthorized, :no_right_on_entity) unless right_on_entity(@instance, true)

          return render json: @instance.errors.messages, status: 422 unless @instance.update(mutate_valid_params)
          render(jsonapi: @instance, serializer: @serializer, status: :ok)
        end

        def destroy
          return render_json_error(:not_found, :instance_not_found) unless @instance
          return render_json_error(:unauthorized, :no_right_on_entity) unless right_on_entity(@instance, true)
          return render(json: @instance.errors, status: :unprocessable_entity) unless @instance.destroy
          
          render(body: nil, status: :no_content)
        end
        
        def show
          return render_json_error(:not_found, :object_not_found) if !@instance || !right_on_entity(@instance, true)
          
          render(
            jsonapi: @instance,
            serializer: @serializer,
            **@options
          )
        end

        def index
          acceptable_params *ACCEPTABLE_PARAMS
          regex_params *REGEX_PARAMS
          valid_operators *VALID_OPERATORS

          aggregation = index_aggregation(no_acl: true)

          if !is_super_admin
            entities = Hermes::UserRight.where(user_id: current_user.id).pluck(:entity_id).uniq
            lessors = Hermes::UserRight.where(user_id: current_user.id).pluck(:lessor_id).uniq
  
            aggregation = aggregation.where(
              '$or': [
                {type: 'entity', :entity_id.in => entities},
                {type: 'lessor', :lessor_id.in => lessors}
              ]
            )
          end

          paginate(
            aggregation,
            each_serializer: @serializer,
            **@options
          )
        end

        def search
          index
        end

        def is_administrator_for_lessor
          is_administrator('lessor')
        end 
        
        def is_administrator_for_entity
          is_administrator('entity')
        end 
        
        def is_administrator(type)
          user = params[:user_id] ? User.find(params[:user_id].to_s) : current_user
          return render_json_error(:not_found, :not_found) unless user
          
          render(
            json: {is_administrator: Hermes::UserRight.where(user: user, type: type, right: 'administrator').count > 0}
          )
        end 

        def delete_bulk
          return render_json_error(:unprocessable_entity, :missing_params) if params[:user_right_ids].nil? || !params[:user_right_ids].is_a?(Array)

          urs = Hermes::UserRight.where(:id.in => params[:user_right_ids])
          return render_json_error(:not_found, :not_found) unless urs.count == params[:user_right_ids].length
          
          all_have_rights = urs.all? { |ur| right_on_entity(ur, true) }
          return render_json_error(:unauthorized, :no_right_on_entity) unless all_have_rights

          urs.each do |ur|
            ur.destroy
          end 

          render(body: nil, status: :no_content)
        end 

        def post_bulk
          return render_json_error(:unprocessable_entity, :missing_params) if params[:user_rights].nil? || !params[:user_rights].is_a?(Array)

          all_have_min_params = params[:user_rights].all? do |user_right|
            user_right[:user_email] && user_right[:type] && user_right[:right] && (user_right[:entity_id] || user_right[:lessor_id])
          end 

          return render_json_error(:unprocessable_entity, :missing_min_params_for_create) unless all_have_min_params

          all_have_rights = params[:user_rights].all? do |user_right|
            right_on_entity(nil, false, user_right)
          end 

          return render_json_error(:unauthorized, :no_right_on_entity) unless all_have_rights

          result = []
          code = 200
          params[:user_rights].each do |data|
            ur = Hermes::UserRight.new(data.symbolize_keys.slice(*MUTATE_PARAMS))
            if ur.save && ur.valid?
              result.push(ur)
            else 
              code = 422
              result = {
                user_emai: data[:user_email],
                errors: ur.errors.messages
              }
              break
            end 
          end 

          if code == 200 
            render(
              jsonapi: result,
              each_serializer: @serializer,
            )
          else 
            render(json: result, code: code)
          end 

        end 

        def delete_all_from_user
          return render_json_error(:unprocessable_entity, :missing_user_id_params) if params[:user_id].blank?

          urs = Hermes::UserRight.where(user_id: params[:user_id])
          all_have_rights = urs.all? { |ur| right_on_entity(ur, true) }
          return render_json_error(:unauthorized, :no_right_on_entity) unless all_have_rights

          urs.each do |ur|
            ur.destroy
          end 

          render(body: nil, status: :no_content)
        end
        
        private 
      end
    end
  end
end
